/*  ---------------------------------------------------------------------------
    Program  :  GetBarInfo.sas

    Author   :  Perry Watts
    Date     :  28Jul2007 15:30
    Project  :  N07 Better Bar Chart Paper (NP16)

    Purpose  :  Obtain values for WIDTH and GSPACE (in Cells) given
                the number of bars, groups, Space (in percent) and GSpace (in
                percent).

    Notes    :  1) This is a macro function. See "Carpenter's Complete Guide to 
                    the Macro Language: Second Edition" for a description of macro
                    functions and instructions for how to use a macro library.

                2) A return value in cells for SPACE is not needed.

                3) Use a scan function to parse the return value e.g:
    %let result=%getBarInfo(Xorigin=17, SpaceSz=2, GspaceSz=2, Ngrps=1, Nbars=3, LCOLfrac=0.75); 
    %let width=%scan(&result,1,':');
    %let gspace=%scan(&result,2,':');

    Parms    :  Xorigin    - in Percent
                SpaceSz    - space between bars (in percent)
                GspaceSz   - space between groups (in percent)
                NGrps      - Number of groups (If midpoint chart, enter 1)
                NBars      - #groups * #midpoints for hierarchical bar chart
                              or a midpoint or subgroup barchart.
                LCOLFrac   - Multiply LCOLS from PROC GDEVICE by 0.01
                             e.g. for EMF = 0.75
    ------------------------------------------------------------------------  */
  %macro getBarInfo(Xorigin=, SpaceSz=, GspaceSz=, Ngrps=, Nbars=, LCOLfrac=);
    %local axislen nGspaces nSpaces width space gspace;
    %let axisLen = %sysevalf(100 - &Xorigin);
    %if &Ngrps le 1 %then %do;
       %let GspaceSz=0; %let Ngrps=1;
    %end;
    %let nGspaces = %sysevalf(&nGrps -1);
    %let nSpaces=%eval(&nbars - &nGspaces - 1);
    %let axisLen = %sysevalf( &axisLen - %sysevalf(&nspaces * &SpaceSz) - %sysevalf(&nGspaces * &GspaceSz));
    %let width = %sysevalf(&axisLen. / &nbars.);
    /* CONVERT FROM PERCENT TO CELLS WITH LCOLFRAC */
    %let space = %sysevalf(&LCOLfrac * &SpaceSz); /*NOT USED FURTHER ON, HOWEVER*/
    %let width=%sysevalf(&LCOLfrac * &width); 
    %let gspace = %sysevalf(&LCOLfrac * &GspaceSz);
    %trim(&width):%trim(&gspace);
  %mend getBarInfo;
