/*---------------------------------------------------------------------------------------
    Program  :  MkSummaryDataBarChart.sas
    Author   :  Perry Watts

    Purpose  :  Create a summary bar chart STAT data set from raw data. 
    Input    :  Data set defined in _INDAT.
    Ouptut   :  Data set defined in _OUTDAT. (See label statement at end for a description
                 of the variables generated.
    Notes    :  Re: RPct for Yvar:
                   For Midpoint charts, each bar adds to 100 pct
                   For Group Charts, bars WITHIN groups add to 100 pct
                   For Subgroup Charts, segments within a bar add to 100 pct
                          
    Parms    :
  Name                      Description                                   Default  Required
 ---------------      ------------------------------------------------    -------  --------
 DATA FILE IDs
  _InDat              Input dataset (including Libname if applies)                    Y
  _OutDat             Output dataset (including Libname) TO BE                        Y 
                       ELIMINATED WITH GSTARS!!!
 ---------------      ------------------------------------------------    -------  --------
 INPUT DATA SET VARIABLES
 *Horizontal Axis:
  _ClassVar           Treatment Variable. The _ClassVar is the argument               Y
                       to the VBAR clause so it plays the role of 
                       the midpoint variable. The midpoint variable
                       determines the bar x-coordinates along the
                       midpoint axis (maxis). It is converted into
                       ClassOrd, a numeric variable inside the macro.
  _ClassDescVar       Treatment Variable Description (for annotation)                 N
                       Assigned to ClassDesc inside the macro. ClassDesc
                       describes ClassOrd. (ordinal ranging from 1 to
                       n).
  _ClassAbbrDescVar   Treatment Variable Description (used only                       N
                       when Group is selected (over CLASS) for coloring
                       and the user wants to display the midpoint
                       axis. If Blank, then filled in with _ClassDescVar.
  _GroupVar           Group Variable - groups the midpoints by a                      N
                       second variable (e.g. time, weight). 
                       Converted into GroupORD, a numeric variable
                       inside the macro.
  _GroupDescVar       Group Variable description (for gAxis).                         N
                       Assigned to GroupDesc inside the macro. GroupDesc
                       describes GroupOrd.              
  _SubGroupVar      SubGroup Variable - for a stacked bar chart.                    N
                       Subordinate to the CLASS variable.
                       Converted into SUBGroupORD, a numeric variable
                       inside the macro.
  _SubGroupDescVar  subGroup Variable description (for the legend)                  N
                       Assigned to subGroupDesc inside the macro. 
                       SubGroupDesc describes subGroupOrd.              
 *Vertical Axis:                           
  _Rvar               Response Variable used as argument to SUMVAR.                   N
                       (SUMVAR NOT used when bc_RespType=FREQ).
-----------------------------------------------------------------------------------------*/

 %macro MkSummaryDataBarChart(_InDat=, _OutDat=, 
  _ClassVar=, _ClassDescVar=, _ClassAbbrDescVar=, _GroupVar=, _GroupDescVar=,
  _SubGroupVar=, _SubGroupDescVar=, _Rvar=);
 
  options nodate number pageno=1 fmtsearch=(work) mautosource;

 /* ADJUST _CLASSDESCVAR AND _CLASSABBRDESCVAR */
  %if &_ClassDescVar eq %then %let _ClassDescVar = &_ClassVar;
  %if &_ClassAbbrDescVar eq %then %let _ClassAbbrDescVar=&_ClassDescVar;

  /* LOAD PARAMETERS FOR HIERARCHY MACRO BELOW. */  
  %if &_SubGroupVar ne %then %do;
    %let H1=CLASS;
    %let H2=SUBGROUP;
  %end;
  %else %do;
    %let H1=GROUP;
    %let H2=CLASS;
  %end;
    
   %macro getORD(Xvar=,XvarDesc=,axisType=);
     %local MAXRNG;
     proc sql noprint;
       create table sqlgrp as
        %if &xVar ne &xVarDesc %then %do;
          select distinct &xVar, &xVarDesc
          from &_InDat
          order by &xVar;
        %end;
        %else %do;
          select distinct &xVarDesc
          from &_InDat
          order by &xVarDesc;
        %end;

       select upcase(TYPE) into :LABELtype
         from dictionary.columns
         where LIBNAME eq 'WORK' and MEMNAME eq 'SQLGRP' and upcase(NAME) = "%upcase(&xVarDesc)";

       select upcase(TYPE) into :RNGtype
         from dictionary.columns
         where LIBNAME eq 'WORK' and MEMNAME eq 'SQLGRP' and upcase(NAME) = "%upcase(&xVar)";

       %if %substr(&RngType,1,1) eq N %then %do;
         select max(&xVar) into :MaxRng
         from sqlgrp;
       %end;
       %else %let MaxRng=.;
     quit;

  /* 
     CLASSORD (WHERE GROUPORD IS PRESENT) MAY BE HIERARCHICAL SUCH THAT IT IS
     REPEATED FOR EACH VALUE OF GROUPORD -- OR NOT. WHEN IT IS NOT, THEN CLASSORD MUST
     BE DERIVED FROM A NUMERIC VERSION OF &_XVAR.
     XVARDESC MAY BE NUMERIC IF A TREATMENT OR GROUP IS IDENTIFIED BY NUMBER ONLY.*/     
     data SqlGrp;
       set Sqlgrp;
       %if &axisType eq MP %then %do;
         /*NOT HIERARCHICAL*/
         %if &XVar ne &xVarDesc AND %substr(&RngType,1,1) eq N AND &MaxRng gt . %then %do;
           n=&Xvar;
         %end;
         /* HIERARCHICAL */
         %else %do;
           n+1;
         %end;
       %end;
       %else %do;
         n+1;
       %end;
       %if %substr(&labelType,1,1) eq N %then %do;
         charDesc=put(&xVarDesc,best.);
       %end;
     run;

     data cntlinN(keep=fmtname type start label);
       length fmtname $8 label $30;
       set sqlGrp;
        fmtname="&axisType.DFm"; type='N'; start=n; 
         %if %substr(&LabelType,1,1) eq C %then %do;
            label=&xVarDesc.; 
         %end;
         %else %do;
            label=charDesc;
         %end;
         output;
     run;
     data cntlinC(keep=fmtname type start label);
       length fmtname $8 start $30 label $2;
       set sqlGrp;
        chN=left(put(n,2.));
        fmtname="&axisType.NFm"; type="%substr(&LabelType,1,1)"; label=chN;
        start=&xVarDesc.; 
        output;
     run;
    
     proc format library=work cntlin=cntlinN;
     run;
     proc format library=work cntlin=cntlinC;
     run;
     %mend getORD;   
     %getOrd(xVar=&_ClassVar, XvarDesc=&_ClassAbbrDescVar, axisType=Mp);
     %if &_GroupVar ne %then %do;
        %getOrd(xVar=&_GroupVar,XvarDesc=&_GroupDescVar, axisType=Grp);
     %end;
     %if &_SubGroupVar ne %then %do;
        %getOrd(xVar=&_SubGroupVar,XvarDesc=&_SubGroupDescVar, axisType=SubG);
     %end;


  /* FIRST STEP AT "DE-MACROIZING".  ONE REC PER PT. GROUP AND SUBGROUP VARIABLES
     ARE MUTUALLY EXCLUSIVE. CAN ONLY HAVE ONE -- NOT BOTH.
   */     
     data indat (Keep = RecNum ClassOrd ClassDesc GroupOrd GroupDesc subGroupOrd 
                        subGroupDesc RVar absRVar);
      retain RecNum ClassOrd ClassDesc GroupOrd GroupDesc subgroupOrd subGroupDesc;
      length ClassDesc GroupDesc subGroupDesc $30;
      set &_InDat;
        RecNum+1;
        RVar=&_Rvar;
        absRvar=abs(Rvar);
        ClassOrd=input(put(&_ClassAbbrDescVar,mpNFm.),2.);            /*TARGET=ORDINAL# */
        ClassDesc=put(ClassOrd,MpDFm.);                               /*INPUT=ORDINAL# TARGET=MIDPOINT DESCRIPTION*/
      %if &_GroupVar ne %then %do;
        GroupOrd=input(put(&_GroupDescVar,grpNFm.),2.);             /*TARGET=ORDINAL#*/
        GroupDesc=put(GroupOrd,GrpDFm.);                              /*INPUT=ORDINAL# TARGET=GROUP DESCRIPTION*/
        subGroupOrd=.;
        subGroupDesc=' ';
      %end;
      %else %if &_SubGroupVar ne %then %do;
        subGroupOrd=input(put(&_SubGroupDescVar,subGNFm.),2.);      /*TARGET=ORDINAL#*/
        subGroupDesc=put(subGroupOrd,subGDFm.);                       /*INPUT=ORDINAL# TARGET=GROUP DESCRIPTION*/
        GroupOrd=.;
        GroupDesc=' ';
     %end;
      %else %do;
        GroupOrd=1;
        GroupDesc=' ';
        subGroupOrd=.;
        subGroupDesc=' ';
      %end;
     run; 

  /* *****
   SUMMARY STATISTICS PROCESSING. Get the following Summary Data:
  ***** */
  %macro hierarchy(H1=, H2=);
   proc summary data=inDat nway;
     class &H1.Ord &H2.Ord;
     var rVar absRVar;
     id &H1.Desc &H2.Desc;
     output out=forEbars mean(rvar)=mean  sum(absRvar)=sum  lclm(rvar)=lclm  uclm(rvar)=uclm 
                         min(rvar)=min  max(rvar)=max 
                         std(rvar)=std  stderr(rvar)=stderr  n(rvar)=&H2.Freq;
   run;
   proc summary data=indat nway;
     class &H1.Ord;
     var rVar absRVar;
     output out=forH1Count n(rvar)=&H1.Freq sum(absRVar)=&H1.Sum;
   run;

   /*CONFIGURE IN THE GRAPHICS MODULE FOR "BOTH" AND "TOP" */
   data forEbars;
     array stats{*} lclm uclm min max histdErr loStdErr hiStd loStd;
     set forEbars;
     hiStdErr=mean+stderr;
     loStdErr=mean-stderr;
     hiStd=mean+std;
     loStd=mean-std;
     do i=1 to dim(stats);
       if stats[i] eq . then stats[i]=mean;
     end;
   run;
   proc sort data=forEbars;
     by &H1.Ord &H2.Ord;
   run;
   proc sort data=forH1Count;
     by &H1.Ord;
   run;   
   data summarydat(keep=groupOrd groupDesc classOrd classDesc subgroupOrd subgroupDesc 
                        mean sum lclm uclm min max hiStdErr loStdErr hiStd loStd Rpct 
                        groupFreq classFreq subGroupFreq);
     %if %upcase(&H1) eq GROUP %then %do;
       retain subgroupOrd subgroupFreq . ;
       retain subgroupDesc ' ';
     %end;
     %else %do;
       retain groupOrd groupFreq .;
       retain groupDesc ' ';
     %end;  
     merge ForEBars forH1Count;
     by &H1.Ord;
     Rpct=100*(sum/&H1.Sum);
   run;

  /* *****
   SUMMARY STATISTICS PROCESSING CONTINUED. Get the following Summary Data:
    CLASSPCT (ADD TO 100% WITHIN A GROUP)
    GROUPPCT (ADD TO 100% FOR EACH CLASS ACROSS GROUPS)
    OAPCT    (SUM OF ALL BARS = 100%
  ***** */
   proc freq data=indat NOPRINT ;
     table &H1.Ord*&H2.Ord / out=bchartStat outpct;
   run;
   data bchartstat (keep=GroupOrd ClassOrd SubGroupOrd ClassPct GroupPct SubGroupPct OAPct);
     set bchartstat;
     %if %upcase(&H1) eq GROUP %then %do;
       retain subgroupOrd subgroupPCT . ;
       retain subgroupDesc ' ';
     %end;
     %else %do;
       retain groupOrd groupPCT .;
       retain groupDesc ' ';
     %end;
     &H1.Pct=input(put(pct_col,5.1),5.1);
     &H2.Pct=input(put(pct_row,5.1),5.1);
     OAPct=input(put(percent,5.1),5.1);
   run;
      
  /* MERGE SUMMARYDAT AND BCHARTSTAT INTO &_OUTDAT. IN GSTARS, &_OUTDAT WILL BE
     REPLACED WITH WORK.INDAT*/
    data &_outDat(keep=ClassOrd ClassDesc GroupOrd GroupDesc subGroupOrd subGroupDesc
       mean sum lclm uclm min max hiStdErr loStdErr hiStd loStd Rpct
       ClassFreq GroupFreq subGroupFreq OAPct mp100pct g100pct sg100pct);
      merge summaryDat bchartstat;
      by GroupOrd ClassOrd;
      if groupDesc ne ' ' then do;
        g100pct=classPct;
        mp100pct=groupPct;
        sg100pct=.;
      end;
      else if subGroupDesc ne ' ' then do;
        mp100pct=subgroupPct;
        sg100pct=classPct;
        g100pct=.;
      end;
      else do; /*have a midpoint chart where OAPct is sufficient*/
        mp100pct=.; sg100pct=.; g100pct=.;
      end;
    run;

    data &_outDat;
     retain ClassOrd ClassDesc GroupOrd GroupDesc subGroupOrd subGroupDesc
       mean sum lclm uclm min max hiStdErr loStdErr hiStd loStd Rpct
       ClassFreq GroupFreq subGroupFreq OAPct mp100pct g100pct sg100pct;
       length classDesc GroupDesc subGroupDesc $30;
       label ClassOrd = 'Class (MP) Number'
           classDesc = 'Class Description'
           GroupOrd = 'Group Number'
           GroupDesc = 'Group Description'
           subGroupOrd= 'Subgroup Number'
           subGroupDesc='Subgroup Description'
           mean='MEAN RVar'
           sum='SUM RVar'
           lclm='LCLM RVar'
           uclm='UCLM RVar'
           min='MIN RVar'
           max='MAX RVar'
           hiStdErr='Upper SEM RVar'
           loStdErr='Lower SEM RVar'
           hiStd='Upper STD RVar'
           loStd='Lower STD RVar'
           Rpct='RVar Percent'
           classFreq='Class (MP) Frequency'
           groupFreq='Group Frequency'
           subgroupFreq='Subgroup Frequency'
           OAPct = 'Overall Percent'
           mp100pct = 'Midpoints add to 100%'
           g100Pct = 'Groups add to 100%'
           sg100Pct = 'Subgroups add to 100%';
       set &_outDat;
   run;

   title1 "&_outdat";  
   proc print noobs data=&_outDat label;
   %if &_groupVar ne %then %do;
    Var GroupOrd GroupDesc ClassOrd ClassDesc 
     mean sum lclm uclm min max hiStdErr loStdErr hiStd loStd Rpct
     GroupFreq ClassFreq g100pct mp100pct OAPct;
   %end;
   %else %if &_subgroupVar ne %then %do;
    var ClassOrd ClassDesc subGroupOrd subGroupDesc
     mean sum lclm uclm min max hiStdErr loStdErr hiStd loStd Rpct
     ClassFreq subGroupFreq mp100pct sg100pct OAPct;
   %end;
   %else %do;
    var ClassOrd ClassDesc
     mean sum lclm uclm min max hiStdErr loStdErr hiStd loStd Rpct
     ClassFreq OAPct;
   %end;
   run;
  %mend hierarchy;
  %hierarchy(H1=&H1, H2=&H2); 
 %MEND MkSummaryDataBarChart;
