/*  ---------------------------------------------------------------------------
    Program  :  OutsideInside.sas
    Author   :  Perry Watts

    Purpose  :  Create an annotate data set that places OUTSIDE and|or INSIDE
                (resizable) text on a bar chart.

    Notes    :  Works only with summary data.
    Input    :  from _indat parameter
    Output   :  data set work.OutInAnno
    Parms    :
  Name                      Description                                  
 ---------------      --------------------------------------------------------   
  _InDat              Input dataset (including Libname if applies)  
  _MPvar              Midpoint variable: Can be character or Numeric.  Must
                       match GCHART midpoint variable.
  _groupVar           Group variable (if applies) Can be character or 
                       Numeric.  Must match GCHART group variable.
  _sgroupVar          Subgroup variable (if applies). Can be character or
                       Numeric.  Must match GCHART subgroup variable.
  _Rvar               Response Variable (e.g. MEAN, CLASSFREQ).
  _outsideVar         Outside variable that supplies text for OUTSIDE option.
  _insideVar          Inside variable that supplies text for INSIDE option.
  _textSize           In percent.
  _outsideFmt         format for OUTSIDE text (typically matches format applied
                       to RVAR in PROC GCHART).
  _insideFmt          format for INSIDE text.

    ------------------------------------------------------------------------  */

 %macro OutsideInside(_indat=, _MPvar=, _groupVar=, _sgroupVar=, _Rvar=, _outsideVar=, 
                     _insideVar=, _textSize=, _outsideFmt=best8., _insideFmt=best8.);

  /* CLEAR OUT DATA SETS FROM PRVIOUS RUNS*/    
    proc datasets lib=work nolist;
      delete plotdat outsideAn insideAn OutInAnno;
    quit;

  /* CREATE PLOTDAT */  
    data plotDat(keep=group midpoint subgroup rvar outside inside);
      set &_indat;
      %if &_groupVar ne %str( ) %then %str(group=&_groupVar;);
      %else %str(group = .;);
      %if &_sgroupVar ne %str( ) %then %do;
        %str(subgroup=&_sgroupVar;);
        %let _outsideVar=%str( );  /*OUTSIDE DOES NOT WORK IN ANNOTATE FOR SUBGROUP CHARTS */
      %end;
      %else %str(subgroup = .;);
       midpoint=&_MPvar;
       %if &_outsideVar ne %str( ) %then %str(outside=&_outsideVar;);
       %if &_insideVar ne %str( ) %then %str(inside=&_insideVar;);
       rvar=&_rvar;
    run;

  /* CREATE ANNOTATE DATA SETS */  
 %if &_OutsideVar ne %then %do;
    data outsideAn;
      length color style text $8;
      retain color 'black' when 'a' style '"Arial"'
             xsys ysys '2' hsys '3' size &_textSize;
      set plotDat;
      if rvar ge 0 OR rvar eq . then position='2';
      else position='8';
      text=left(put(outside,&_outsideFmt));
    run;
  %end;
  %if &_InsideVar ne %then %do;
    data InsideAn;
      length color style text $8 position $1;
      retain color 'black' when 'a' style '"Arial"'
             xsys ysys '2' hsys '3' size &_textSize;
      set plotDat;
      if subgroup ne . then do;
        if inside gt 0;
        if rvar gt 0 then position='E'; /*half cell below*/
        else position='B';              /*half cell above*/
      end;
      else do;
        if rvar gt 0 then position='E'; /*half cell below*/
        else position='2';              /*one cell above*/
      end;
      if outside eq . and inside eq 0 then position='2';
      text=left(put(inside,&_insideFmt));
    run;
  %end;
  %if &_OutsideVar ne %str( ) AND _InsideVar ne %str( ) %then %let SetNames= OutsideAn InsideAn;
  %else %if &_OutsideVar ne %str( ) %then %let setNames = OutsideAn;
  %else %let setNames= InsideAn;

    data OutInAnno;
      set &setNames;
    run;
%mend outsideInside;
