/*  ---------------------------------------------------------------------------
    Program  :  displayEbars.sas
    Author   :  Perry Watts
    Project  :  N07 Better Bar Chart Paper (NP16)

    Purpose  :  Create error bars for bar charts via macro.

    Notes    :  Works only with summary data.
                Designed for the EMF device.
                Subgroup Charts cannot have error bars.

    Input    :  from _indat parameter
    Output   :  data set work.EBarAnno
    Parms    :
  Name                      Description                                  
 ---------------      --------------------------------------------------------   
  _InDat              Input dataset (including Libname if applies)  
  _MPvar              Midpoint variable: Can be character or Numeric.  Must
                       match GCHART midpoint variable.
  _groupVar           Group variable (if applies) Can be character or 
                       Numeric.  Must match GCHART group variable.
  _Rvar               Response Variable (Typically MEAN)
  _loY                Low-Range variable
  _hiY                High Range Variable
  _fatLine            In percent - for the wide tick widths at tops and bottoms
                       of error bars. For EMF device, &width in CELLS is used
                       for the width in PERCENT of the bars.
  _thinLine           In percent - for the width of the error bar line itself.
                       _ThinLine is also used to calculate the vertical HEIGHT
                       of the _fatLine tick. (Range generally is 0.5 to 2).
                       With EMF device wide ticks can be so short that they
                       are not visible on the screen (print is OK). CGM does
                       not have this problem.
  _EbarType           TOP or BOTH -- same as GCHART except that bars plot
                       as non-existent BOTTOM when values are negative.
    ------------------------------------------------------------------------  */

%macro displayEbars(_indat=, _MPvar=, _groupVar=, _Rvar=, _loY=, _hiY=,
                    _fatLine=&width, _thinLine=0.5, _EbarType=TOP);
    %if &_groupVar ne %str( ) %then
      %let keep=%str(xsys ysys hsys function size line x midpoint y color group when);
    %else
      %let keep=%str(xsys ysys hsys function size line x midpoint y color when);
    %let fuzz = &_thinline;
    %let fuzzdiv2=%sysevalf(&fuzz / 2);

    data EBarAnno(keep=&keep);
      length function color $8;
      retain fuzz &fuzz fuzzdiv2 &fuzzdiv2 thinLine &_thinLine fatLine &_fatLine;
      retain xsys '2' hsys '3' when 'a' line 1 color 'black';
      set &_indat end=last;
      %if &_groupVar ne %str( ) %then group=&_groupVar; ;
      midpoint=&_MPvar;
      loY=&_loY; hiY=&_hiY; Rvar=&_Rvar;
     %if &_EbarType eq TOP %then %do;
       if Rvar ge 0 then loY=RVar;
       else hiY=Rvar;
     %end;
      if loY ne . and hiY ne . then do;
       function='move';
      %if &_EbarType eq TOP %then %do;
       if hiY gt 0 then do;
        ysys='2'; y=loY; output; /*Error bars go up. Start from bottom (top of positive bar)*/
         function='draw'; size=thinLine; y=hiy; output;
        ysys='9'; function='move'; y=-fuzzdiv2; output; 
         function='draw'; size=fatLine; y=+fuzz; output;
       end;
       else do; /*Error bars go down. Start from "top". (bottom of negative bar)*/
        ysys='2'; y=hiY; output;  
         function='draw'; size=thinLine; y=loy; output;
        ysys='9'; function='move'; y=+fuzzdiv2; output; 
         function='draw'; size=fatLine; y=-fuzz; output;
       end;
      %end;
      %else %do; /*BOTH*/
        ysys='2'; Y=rVar; output; /*Error bar goes up. Start from MEAN. */
         function='draw'; size=thinLine; y=hiy; output;
        ysys='9'; function='move'; y=-fuzzdiv2; output; 
         function='draw'; size=fatLine; y=+fuzz; output;
       /*Go back to the MEAN and then plot the lower half of the E Bar */
        ysys='2'; function='move'; Y=rVar; output; 
         function='draw'; size=thinLine; y=loy; output;
        ysys='9'; function='move'; y=+fuzzdiv2; output; 
         function='draw'; size=fatLine; y=-fuzz; output;
      %end;
     end;
     if last then do;
       xsys='1'; ysys='2'; x=0; y=0; function='move'; output;
       function='draw'; size=0.5; x=100; output;
     end;
    run;
%mend displayEbars;
